@@ -71,6 +71,79 @@ char *color_to_string(struct color c, char buf[10])
         return buf;
 }
 
+struct gradient *gradient_alloc(size_t length)
+{
+        if (length == 0)
+                return NULL;
+
+        struct gradient *grad = g_rc_box_alloc(sizeof(struct gradient) + length * sizeof(struct color));
+
+        grad->length = length;
+        grad->pattern = NULL;
+
+        return grad;
+}
+
+struct gradient *gradient_acquire(struct gradient *grad)
+{
+        return grad != NULL ? g_rc_box_acquire(grad) : NULL;
+}
+
+static void gradient_free(struct gradient *grad)
+{
+        if (grad->pattern)
+                cairo_pattern_destroy(grad->pattern);
+}
+
+void gradient_release(struct gradient *grad)
+{
+        if (grad != NULL)
+                g_rc_box_release_full(grad, (GDestroyNotify)gradient_free);
+}
+
+void gradient_pattern(struct gradient *grad)
+{
+        if (grad->length == 1) {
+                grad->pattern = cairo_pattern_create_rgba(grad->colors[0].r,
+                                                          grad->colors[0].g,
+                                                          grad->colors[0].b,
+                                                          grad->colors[0].a);
+        } else {
+                grad->pattern = cairo_pattern_create_linear(0, 0, 1, 0);
+                for (size_t i = 0; i < grad->length; i++) {
+                        double offset = i  / (double)(grad->length - 1);
+                        cairo_pattern_add_color_stop_rgba(grad->pattern,
+                                                          offset,
+                                                          grad->colors[i].r,
+                                                          grad->colors[i].g,
+                                                          grad->colors[i].b,
+                                                          grad->colors[i].a);
+                }
+        }
+}
+
+char *gradient_to_string(const struct gradient *grad)
+{
+        if (!GRADIENT_VALID(grad)) return NULL;
+
+        int max = grad->length * 11 + 1;
+        char *buf = g_malloc(max);
+
+        for (size_t i = 0, j = 0; i < grad->length; i++) {
+                j += g_snprintf(buf + j, max - j, "#%02x%02x%02x%02x",
+                                  (int)(grad->colors[i].r * 255),
+                                  (int)(grad->colors[i].g * 255),
+                                  (int)(grad->colors[i].b * 255),
+                                  (int)(grad->colors[i].a * 255));
+
+                if (i != grad->length - 1) {
+                        j += g_snprintf(buf + j, max - j, ", ");
+                }
+        }
+
+        return buf;
+}
+
 void draw_setup(void)
 {
         const struct output *out = output_create(settings.force_xwayland);
@@ -202,7 +275,7 @@ static void layout_setup(struct colored_layout *cl, int width, int height, doubl
         int icon_width = cl->icon ? get_icon_width(cl->icon, scale) + horizontal_padding : 0;
         int text_width = width - 2 * settings.h_padding - (cl->n->icon_position == ICON_TOP ? 0 : icon_width);
         int progress_bar_height = have_progress_bar(cl) ? settings.progress_bar_height + settings.padding : 0;
-        int max_text_height = MAX(0, settings.height - progress_bar_height - 2 * settings.padding);
+        int max_text_height = MAX(0, settings.height.max - progress_bar_height - 2 * settings.padding);
         layout_setup_pango(cl->l, text_width, max_text_height, cl->n->word_wrap, cl->n->ellipsize, cl->n->alignment);
 }
 
@@ -219,7 +292,7 @@ static void free_colored_layout(void *data)
 static struct dimensions calculate_notification_dimensions(struct colored_layout *cl, double scale)
 {
         struct dimensions dim = { 0 };
-        layout_setup(cl, settings.width.max, settings.height, scale);
+        layout_setup(cl, settings.width.max, settings.height.max, scale);
 
         int horizontal_padding = get_horizontal_text_icon_padding(cl->n);
         int icon_width = cl->icon? get_icon_width(cl->icon, scale) + horizontal_padding : 0;
@@ -242,14 +315,16 @@ static struct dimensions calculate_notification_dimensions(struct colored_layout
                 dim.h = MAX(icon_height, dim.text_height);
         }
 
-        dim.h += progress_bar_height;
+        dim.h += progress_bar_height + settings.padding * 2;
         dim.w = dim.text_width + icon_width + 2 * settings.h_padding;
 
-        dim.h = MIN(settings.height, dim.h + settings.padding * 2);
-        dim.w = MAX(settings.width.min, dim.w);
         if (have_progress_bar(cl))
                 dim.w = MAX(settings.progress_bar_min_width, dim.w);
 
+        dim.h = MAX(settings.height.min, dim.h);
+        dim.h = MIN(settings.height.max, dim.h);
+
+        dim.w = MAX(settings.width.min, dim.w);
         dim.w = MIN(settings.width.max, dim.w);
 
         cl->n->displayed_height = dim.h;
@@ -318,7 +393,7 @@ static struct colored_layout *layout_init_shared(cairo_t *c, struct notification
 
         // Invalid colors should never reach this point!
         assert(settings.frame_width == 0 || COLOR_VALID(COLOR(cl, frame)));
-        assert(!have_progress_bar(cl) || COLOR_VALID(COLOR(cl, highlight)));
+        assert(!have_progress_bar(cl) || COLOR(cl, highlight) != NULL);
         assert(COLOR_VALID(COLOR(cl, fg)));
         assert(COLOR_VALID(COLOR(cl, bg)));
         return cl;
@@ -425,12 +500,9 @@ static int layout_get_height(struct colored_layout *cl, double scale)
                 h_progress_bar = settings.progress_bar_height + settings.padding;
         }
 
-
-        if (cl->n->icon_position == ICON_TOP && cl->n->icon) {
-                return h_icon + h_text + h_progress_bar + vertical_padding;
-        } else {
-                return MAX(h_text, h_icon) + h_progress_bar;
-        }
+        return (cl->n->icon_position == ICON_TOP && cl->n->icon)
+                ? h_icon + h_text + h_progress_bar + vertical_padding
+                : MAX(h_text, h_icon) + h_progress_bar;
 }
 
 /* Attempt to make internal radius more organic.
@@ -705,73 +777,74 @@ static cairo_surface_t *render_background(cairo_surface_t *srf,
                                                   round(width * scale), round(height * scale));
 }
 
-static void render_content(cairo_t *c, struct colored_layout *cl, int width, double scale)
+static void render_content(cairo_t *c, struct colored_layout *cl, int width, int height, double scale)
 {
         // Redo layout setup, while knowing the width. This is to make
         // alignment work correctly
-        layout_setup(cl, width, settings.height, scale);
+        layout_setup(cl, width, height, scale);
 
-        const int h = layout_get_height(cl, scale);
-        LOG_D("Layout height %i", h);
-        int h_without_progress_bar = h;
+        // NOTE: Includes paddings!
+        int h_without_progress_bar = height;
         if (have_progress_bar(cl)) {
-                 h_without_progress_bar -= settings.progress_bar_height + settings.padding;
+                h_without_progress_bar -= settings.progress_bar_height + settings.padding;
         }
 
+        int text_h = 0;
         if (!cl->n->hide_text) {
-                int h_text = 0;
-                get_text_size(cl->l, NULL, &h_text, scale);
-
-                int text_x = settings.h_padding,
-                    text_y = settings.padding + h_without_progress_bar / 2 - h_text / 2;
-
-                // text positioning
-                if (cl->icon) {
-                        // vertical alignment
-                        if (settings.vertical_alignment == VERTICAL_TOP) {
-                                text_y = settings.padding;
-                        } else if (settings.vertical_alignment == VERTICAL_BOTTOM) {
-                                text_y = h_without_progress_bar + settings.padding - h_text;
-                                if (text_y < 0)
-                                        text_y = settings.padding;
-                        } // else VERTICAL_CENTER
-
-                        // icon position
-                        if (cl->n->icon_position == ICON_LEFT) {
-                                text_x = get_icon_width(cl->icon, scale) + settings.h_padding + get_horizontal_text_icon_padding(cl->n);
-                        } else if (cl->n->icon_position == ICON_TOP) {
-                                text_y = get_icon_height(cl->icon, scale) + settings.padding + get_vertical_text_icon_padding(cl->n);
-                        } // else ICON_RIGHT
-                }
-                cairo_move_to(c, round(text_x * scale), round(text_y * scale));
-
-                cairo_set_source_rgba(c, COLOR(cl, fg.r), COLOR(cl, fg.g), COLOR(cl, fg.b), COLOR(cl, fg.a));
-                pango_cairo_update_layout(c, cl->l);
-                pango_cairo_show_layout(c, cl->l);
+                get_text_size(cl->l, NULL, &text_h, scale);
         }
 
+        // text vertical alignment
+        int text_x = settings.h_padding,
+            text_y = settings.padding;
+
+        if (settings.vertical_alignment == VERTICAL_CENTER) {
+                text_y = h_without_progress_bar / 2 - text_h / 2;
+        } else if (settings.vertical_alignment == VERTICAL_BOTTOM) {
+                text_y = h_without_progress_bar - settings.padding - text_h;
+                if (text_y < 0) text_y = settings.padding;
+        } // else VERTICAL_TOP
+
         // icon positioning
-        if (cl->icon) {
-                unsigned int image_width = get_icon_width(cl->icon, scale),
-                             image_height = get_icon_height(cl->icon, scale),
-                             image_x = width - settings.h_padding - image_width,
-                             image_y = settings.padding + h_without_progress_bar/2 - image_height/2;
+        if (cl->icon && cl->n->icon_position != ICON_OFF) {
+                int image_width = get_icon_width(cl->icon, scale),
+                    image_height = get_icon_height(cl->icon, scale),
+                    image_x = width - settings.h_padding - image_width,
+                    image_y = text_y,
+                    v_padding = get_vertical_text_icon_padding(cl->n);
 
                 // vertical alignment
-                if (settings.vertical_alignment == VERTICAL_TOP) {
-                        image_y = settings.padding;
-                } else if (settings.vertical_alignment == VERTICAL_BOTTOM) {
-                        image_y = h_without_progress_bar + settings.padding - image_height;
-                        if (image_y < settings.padding || image_y > h_without_progress_bar)
-                                image_y = settings.padding;
-                } // else VERTICAL_CENTER
+                switch (settings.vertical_alignment) {
+                        case VERTICAL_TOP:
+                                if (cl->n->icon_position == ICON_TOP) {
+                                        // Shift text downward
+                                        text_y += image_height + v_padding;
+                                }
+                                break;
+                        case VERTICAL_CENTER:
+                                if (cl->n->icon_position == ICON_TOP) {
+                                        // Adjust text and image by half
+                                        image_y -= (image_height + v_padding) / 2;
+                                        text_y += (image_height + v_padding) / 2;
+                                } else {
+                                        image_y += text_h / 2 - image_height / 2;
+                                }
+                                break;
+                        case VERTICAL_BOTTOM:
+                                if (cl->n->icon_position == ICON_TOP) {
+                                        image_y -= image_height + v_padding;
+                                } else {
+                                        image_y -= image_height - text_h;
+                                }
+                                break;
+                }
 
                 // icon position
-                if (cl->n->icon_position == ICON_LEFT) {
+                if (cl->n->icon_position == ICON_TOP) {
+                        image_x = (width - image_width) / 2;
+                } else if (cl->n->icon_position == ICON_LEFT) {
                         image_x = settings.h_padding;
-                } else if (cl->n->icon_position == ICON_TOP) {
-                        image_y = settings.padding;
-                        image_x = width/2 - image_width/2;
+                        text_x += image_width + get_horizontal_text_icon_padding(cl->n);
                 } // else ICON_RIGHT
 
                 cairo_set_source_surface(c, cl->icon, round(image_x * scale), round(image_y * scale));
@@ -779,6 +852,14 @@ static void render_content(cairo_t *c, struct colored_layout *cl, int width, dou
                 cairo_fill(c);
         }
 
+        // text positioning
+        if (!cl->n->hide_text) {
+                cairo_move_to(c, round(text_x * scale), round(text_y * scale));
+                cairo_set_source_rgba(c, COLOR(cl, fg.r), COLOR(cl, fg.g), COLOR(cl, fg.b), COLOR(cl, fg.a));
+                pango_cairo_update_layout(c, cl->l);
+                pango_cairo_show_layout(c, cl->l);
+        }
+
         // progress bar positioning
         if (have_progress_bar(cl)) {
                 int progress = MIN(cl->n->progress, 100);
@@ -786,10 +867,11 @@ static void render_content(cairo_t *c, struct colored_layout *cl, int width, dou
                 unsigned int frame_width = settings.progress_bar_frame_width,
                              progress_width = MIN(width - 2 * settings.h_padding, settings.progress_bar_max_width),
                              progress_height = settings.progress_bar_height - frame_width,
-                             frame_y = settings.padding + h - settings.progress_bar_height,
+                             frame_y = h_without_progress_bar,
                              progress_width_without_frame = progress_width - 2 * frame_width,
                              progress_width_1 = progress_width_without_frame * progress / 100,
-                             progress_width_2 = progress_width_without_frame - 1;
+                             progress_width_2 = progress_width_without_frame - 1,
+                             progress_width_scaled = (progress_width + 1) * scale;
 
                 switch (cl->n->progress_bar_alignment) {
                         case PANGO_ALIGN_LEFT:
@@ -820,7 +902,11 @@ static void render_content(cairo_t *c, struct colored_layout *cl, int width, dou
                 cairo_fill(c);
 
                 // top layer (fill)
-                cairo_set_source_rgba(c, COLOR(cl, highlight.r), COLOR(cl, highlight.g), COLOR(cl, highlight.b), COLOR(cl, highlight.a));
+                cairo_matrix_t matrix;
+                cairo_matrix_init_scale(&matrix, 1.0 / progress_width_scaled, 1.0);
+                cairo_pattern_set_matrix(COLOR(cl, highlight->pattern), &matrix);
+                cairo_set_source(c, COLOR(cl, highlight->pattern));
+
                 draw_rounded_rect(c, x_bar_1, frame_y, progress_width_1, progress_height,
                         settings.progress_bar_corner_radius, scale, settings.progress_bar_corners);
                 cairo_fill(c);
@@ -848,16 +934,14 @@ static struct dimensions layout_render(cairo_surface_t *srf,
         double scale = output->get_scale();
         const int cl_h = layout_get_height(cl, scale);
 
-        int h_text = 0;
-        get_text_size(cl->l, NULL, &h_text, scale);
-
         int bg_width = 0;
-        int bg_height = MIN(settings.height, (2 * settings.padding) + cl_h);
+        int bg_height = MAX(settings.height.min, 2 * settings.padding + cl_h);
+        bg_height = MIN(settings.height.max, bg_height);
 
         cairo_surface_t *content = render_background(srf, cl, cl_next, dim.y, dim.w, bg_height, dim.corner_radius, corners, &bg_width, scale);
         cairo_t *c = cairo_create(content);
 
-        render_content(c, cl, bg_width, scale);
+        render_content(c, cl, bg_width, bg_height, scale);
 
         /* adding frame */
         if (corners & (C_TOP | _C_FIRST))
@@ -866,10 +950,7 @@ static struct dimensions layout_render(cairo_surface_t *srf,
         if (corners & (C_BOT | _C_LAST))
                 dim.y += settings.frame_width;
 
-        if ((2 * settings.padding + cl_h) < settings.height)
-                dim.y += cl_h + 2 * settings.padding;
-        else
-                dim.y += settings.height;
+        dim.y += bg_height;
 
         if (settings.gap_size)
                 dim.y += settings.gap_size;
@@ -974,6 +1055,7 @@ void draw(void)
 
 void draw_deinit(void)
 {
+        pango_font_description_free(pango_fdesc);
         output->win_destroy(win);
         output->deinit();
         if (settings.enable_recursive_icon_lookup)
