@@ -25,6 +25,8 @@ enum dunst_status_field {
         S_PAUSE_LEVEL,
 };
 
+extern char **config_paths;
+
 /**
  * Modify the current status of dunst
  * @param field The field to change in the global status structure
@@ -38,6 +40,7 @@ void dunst_status_int(const enum dunst_status_field field,
 struct dunst_status dunst_status_get(void);
 
 void wake_up(void);
+void reload(char **const configs);
 
 int dunst_main(int argc, char *argv[]);
 
