@@ -97,7 +97,7 @@ bool queues_notification_replace_id(struct notification *new);
  * @post Call wake_up() to synchronize the queues with the UI
  *       (which closes the notification on screen)
  */
-void queues_notification_close_id(int id, enum reason reason);
+void queues_notification_close_id(gint id, enum reason reason);
 
 /**
  * Close the given notification. \see queues_notification_close_id().
@@ -109,8 +109,9 @@ void queues_notification_close(struct notification *n, enum reason reason);
 
 /**
  * Removes all notifications from history
+ * Returns the number of removed notifications
  */
-void queues_history_clear(void);
+guint queues_history_clear(void);
 
 /**
  * Pushes the latest notification of history to the displayed queue
@@ -122,7 +123,7 @@ void queues_history_pop(void);
  * Pushes the latest notification found in the history buffer identified by
  * it's assigned id
  */
-void queues_history_pop_by_id(unsigned int id);
+void queues_history_pop_by_id(gint id);
 
 /**
  * Push a single notification to history
@@ -138,9 +139,9 @@ void queues_history_push(struct notification *n);
 void queues_history_push_all(void);
 
 /**
- * Removes an notification identified by the given id from the history 
+ * Removes an notification identified by the given id from the history
  */
-void queues_history_remove_by_id(unsigned int id);
+bool queues_history_remove_by_id(gint id);
 
 /**
  * Move inserted notifications from waiting queue to displayed queue
@@ -177,8 +178,12 @@ gint64 queues_get_next_datachange(gint64 time);
  *
  * @return the `id` notification  or NULL
  */
-struct notification* queues_get_by_id(int id);
+struct notification* queues_get_by_id(gint id);
 
+/**
+ * Reapply all rules to the queue (used when reloading configs)
+ */
+void queues_reapply_all_rules(void);
 
 /**
  * Remove all notifications from all list and free the notifications
