@@ -332,10 +332,10 @@ void wl_deinit(void) {
         // could have been aborted half way through, or the compositor doesn't
         // support some of these features.
         if (ctx.layer_surface != NULL) {
-                zwlr_layer_surface_v1_destroy(ctx.layer_surface);
+                g_clear_pointer(&ctx.layer_surface, zwlr_layer_surface_v1_destroy);
         }
         if (ctx.surface != NULL) {
-                wl_surface_destroy(ctx.surface);
+                g_clear_pointer(&ctx.surface, wl_surface_destroy);
         }
         finish_buffer(&ctx.buffers[0]);
         finish_buffer(&ctx.buffers[1]);
@@ -352,38 +352,41 @@ void wl_deinit(void) {
                 destroy_seat(seat);
         }
 
+        ctx.outputs = (struct wl_list) {0};
+        ctx.seats = (struct wl_list) {0};
+
 #ifdef HAVE_WL_CURSOR_SHAPE
         if (ctx.cursor_shape_manager)
-                wp_cursor_shape_manager_v1_destroy(ctx.cursor_shape_manager);
+                g_clear_pointer(&ctx.cursor_shape_manager, wp_cursor_shape_manager_v1_destroy);
 #endif
 
 #ifdef HAVE_WL_EXT_IDLE_NOTIFY
         if (ctx.ext_idle_notifier)
-                ext_idle_notifier_v1_destroy(ctx.ext_idle_notifier);
+                g_clear_pointer(&ctx.ext_idle_notifier, ext_idle_notifier_v1_destroy);
 #endif
 
         if (ctx.idle_handler)
-                org_kde_kwin_idle_destroy(ctx.idle_handler);
+                g_clear_pointer(&ctx.idle_handler, org_kde_kwin_idle_destroy);
 
         if (ctx.layer_shell)
-                zwlr_layer_shell_v1_destroy(ctx.layer_shell);
+                g_clear_pointer(&ctx.layer_shell, zwlr_layer_shell_v1_destroy);
 
         if (ctx.compositor)
-                wl_compositor_destroy(ctx.compositor);
+                g_clear_pointer(&ctx.compositor, wl_compositor_destroy);
 
         if (ctx.shm)
-                wl_shm_destroy(ctx.shm);
+                g_clear_pointer(&ctx.shm, wl_shm_destroy);
 
         if (ctx.registry)
-                wl_registry_destroy(ctx.registry);
+                g_clear_pointer(&ctx.registry, wl_registry_destroy);
 
         if (ctx.cursor_theme != NULL) {
-                wl_cursor_theme_destroy(ctx.cursor_theme);
-                wl_surface_destroy(ctx.cursor_surface);
+                g_clear_pointer(&ctx.cursor_theme, wl_cursor_theme_destroy);
+                g_clear_pointer(&ctx.cursor_surface, wl_surface_destroy);
         }
 
         // this also disconnects the wl_display
-        g_water_wayland_source_free(ctx.esrc);
+        g_clear_pointer(&ctx.esrc, g_water_wayland_source_free);
 }
 
 static void schedule_frame_and_commit(void);
@@ -693,7 +696,7 @@ double wl_get_scale(void) {
                 // return the largest scale
                 struct dunst_output *output;
                 wl_list_for_each(output, &ctx.outputs, link) {
-                        scale = MAX(output->scale, scale);
+                        scale = MAX((int)output->scale, scale);
                 }
         }
         if (scale <= 0)
