@@ -7,21 +7,12 @@
 
 #define INRECT(x,y,rx,ry,rw,rh) ((x) >= (rx) && (x) < (rx)+(rw) && (y) >= (ry) && (y) < (ry)+(rh))
 
-struct screen_info {
-        int id;
-        int x;
-        int y;
-        unsigned int h;
-        unsigned int mmh;
-        unsigned int w;
-};
-
 void init_screens(void);
 void screen_dpi_xft_cache_purge(void);
 bool screen_check_event(XEvent *ev);
 
-struct screen_info *get_active_screen(void);
-double screen_dpi_get(struct screen_info *scr);
+const struct screen_info *get_active_screen(void);
+double screen_dpi_get(const struct screen_info *scr);
 
 /**
  * Find the currently focused window and check if it's in
@@ -45,4 +36,4 @@ bool have_fullscreen_window(void);
 bool window_is_fullscreen(Window window);
 
 #endif
-/* vim: set tabstop=8 shiftwidth=8 expandtab textwidth=0: */
+/* vim: set ft=c tabstop=8 shiftwidth=8 expandtab textwidth=0: */
