@@ -19,7 +19,7 @@ const struct screen_info* noop_screen(void) {
 }
 
 const struct output dummy_output = {
-#if WAYLAND_ONLY
+#if !X11_SUPPORT
         wl_init,
         wl_deinit,
 
@@ -70,23 +70,23 @@ GSList *get_dummy_layouts(GSList *notifications)
         return layouts;
 }
 
-int get_small_max_height(void)
+struct length get_small_max_height(void)
 {
         // to keep test calculations simpler, set max height small to
         // only test cases where height is not dynamically determined
         // by notification content
         // future tests targeting dynamic sizing logic could be added
         // to address this limitation
-        int small_max_height = 10;
-        return small_max_height;
+		struct length height = { 0, 10 };
+		return height;
 }
 
-int get_expected_dimension_height(int layout_count)
+int get_expected_dimension_height(int layout_count, int height)
 {
-        // assumes settings.height == notification height, see get_small_max_height
+        // assumes height == notification height, see get_small_max_height
         int separator_height = (layout_count - 1) * settings.separator_height;
         int total_gap_size = (layout_count - 1) * settings.gap_size;
-        int height = settings.height * layout_count;
+        height *= layout_count;
         int frame_width_total_height;
         int expected_height;
         if(settings.gap_size) {
@@ -102,7 +102,7 @@ int get_expected_dimension_height(int layout_count)
 int get_expected_dimension_y_offset(int layout_count)
 {
         // assumes settings.height == notification height, see get_small_max_height
-        int expected_y = layout_count * settings.height;
+        int expected_y = layout_count * settings.height.max;
         if(settings.gap_size) {
                 expected_y += (layout_count * (2 * settings.frame_width));
                 expected_y += (layout_count * settings.gap_size);
@@ -155,7 +155,7 @@ TEST test_layout_from_notification_no_icon(void)
 
 TEST test_calculate_dimensions_height_no_gaps(void)
 {
-        int original_height = settings.height;
+        struct length original_height = settings.height;
         bool orginal_gap_size = settings.gap_size;
         settings.height = get_small_max_height();
         settings.gap_size = 10;
@@ -170,7 +170,7 @@ TEST test_calculate_dimensions_height_no_gaps(void)
         notifications = get_dummy_notifications(layout_count);
         layouts = get_dummy_layouts(notifications);
         dim = calculate_dimensions(layouts);
-        expected_height = get_expected_dimension_height(layout_count);
+        expected_height = get_expected_dimension_height(layout_count, settings.height.max);
         ASSERT(dim.h == expected_height);
         g_slist_free_full(layouts, free_colored_layout);
         g_slist_free_full(notifications, free_dummy_notification);
@@ -179,7 +179,7 @@ TEST test_calculate_dimensions_height_no_gaps(void)
         notifications = get_dummy_notifications(layout_count);
         layouts = get_dummy_layouts(notifications);
         dim = calculate_dimensions(layouts);
-        expected_height = get_expected_dimension_height(layout_count);
+        expected_height = get_expected_dimension_height(layout_count, settings.height.max);
         ASSERT(dim.h == expected_height);
         g_slist_free_full(layouts, free_colored_layout);
         g_slist_free_full(notifications, free_dummy_notification);
@@ -188,7 +188,7 @@ TEST test_calculate_dimensions_height_no_gaps(void)
         notifications = get_dummy_notifications(layout_count);
         layouts = get_dummy_layouts(notifications);
         dim = calculate_dimensions(layouts);
-        expected_height = get_expected_dimension_height(layout_count);
+        expected_height = get_expected_dimension_height(layout_count, settings.height.max);
         ASSERT(dim.h == expected_height);
         g_slist_free_full(layouts, free_colored_layout);
         g_slist_free_full(notifications, free_dummy_notification);
@@ -201,7 +201,7 @@ TEST test_calculate_dimensions_height_no_gaps(void)
 
 TEST test_calculate_dimensions_height_gaps(void)
 {
-        int original_height = settings.height;
+        struct length original_height = settings.height;
         bool orginal_gap_size = settings.gap_size;
         settings.height = get_small_max_height();
         settings.gap_size = 10;
@@ -216,7 +216,7 @@ TEST test_calculate_dimensions_height_gaps(void)
         notifications = get_dummy_notifications(layout_count);
         layouts = get_dummy_layouts(notifications);
         dim = calculate_dimensions(layouts);
-        expected_height = get_expected_dimension_height(layout_count);
+        expected_height = get_expected_dimension_height(layout_count, settings.height.max);
         ASSERT(dim.h == expected_height);
         g_slist_free_full(layouts, free_colored_layout);
         g_slist_free_full(notifications, free_dummy_notification);
@@ -225,7 +225,7 @@ TEST test_calculate_dimensions_height_gaps(void)
         notifications = get_dummy_notifications(layout_count);
         layouts = get_dummy_layouts(notifications);
         dim = calculate_dimensions(layouts);
-        expected_height = get_expected_dimension_height(layout_count);
+        expected_height = get_expected_dimension_height(layout_count, settings.height.max);
         ASSERT(dim.h == expected_height);
         g_slist_free_full(layouts, free_colored_layout);
         g_slist_free_full(notifications, free_dummy_notification);
@@ -234,7 +234,7 @@ TEST test_calculate_dimensions_height_gaps(void)
         notifications = get_dummy_notifications(layout_count);
         layouts = get_dummy_layouts(notifications);
         dim = calculate_dimensions(layouts);
-        expected_height = get_expected_dimension_height(layout_count);
+        expected_height = get_expected_dimension_height(layout_count, settings.height.max);
         ASSERT(dim.h == expected_height);
 
         g_slist_free_full(layouts, free_colored_layout);
@@ -247,7 +247,7 @@ TEST test_calculate_dimensions_height_gaps(void)
 
 TEST test_layout_render_no_gaps(void)
 {
-        int original_height = settings.height;
+        struct length original_height = settings.height;
         bool orginal_gap_size = settings.gap_size;
         settings.height = get_small_max_height();
         settings.gap_size = 0;
@@ -289,9 +289,57 @@ TEST test_layout_render_no_gaps(void)
         PASS();
 }
 
+TEST test_calculate_dimensions_height_min(void)
+{
+        struct length original_height = settings.height;
+        bool orginal_gap_size = settings.gap_size;
+		// NOTE: Should be big enough to fit the notification nicely
+        settings.height.min = 100;
+        settings.height.max = 200;
+        settings.gap_size = 0;
+
+        int layout_count;
+        GSList *notifications;
+        GSList *layouts;
+        struct dimensions dim;
+        int expected_height;
+
+        layout_count = 1;
+        notifications = get_dummy_notifications(layout_count);
+        layouts = get_dummy_layouts(notifications);
+        dim = calculate_dimensions(layouts);
+        expected_height = get_expected_dimension_height(layout_count, settings.height.min);
+        ASSERT(dim.h == expected_height);
+        g_slist_free_full(layouts, free_colored_layout);
+        g_slist_free_full(notifications, free_dummy_notification);
+
+        layout_count = 2;
+        notifications = get_dummy_notifications(layout_count);
+        layouts = get_dummy_layouts(notifications);
+        dim = calculate_dimensions(layouts);
+        expected_height = get_expected_dimension_height(layout_count, settings.height.min);
+        ASSERT(dim.h == expected_height);
+        g_slist_free_full(layouts, free_colored_layout);
+        g_slist_free_full(notifications, free_dummy_notification);
+
+        layout_count = 3;
+        notifications = get_dummy_notifications(layout_count);
+        layouts = get_dummy_layouts(notifications);
+        dim = calculate_dimensions(layouts);
+        expected_height = get_expected_dimension_height(layout_count, settings.height.min);
+        ASSERT(dim.h == expected_height);
+
+        g_slist_free_full(layouts, free_colored_layout);
+        g_slist_free_full(notifications, free_dummy_notification);
+        settings.gap_size = orginal_gap_size;
+        settings.height = original_height;
+
+        PASS();
+}
+
 TEST test_layout_render_gaps(void)
 {
-        int original_height = settings.height;
+        struct length original_height = settings.height;
         bool orginal_gap_size = settings.gap_size;
         settings.height = get_small_max_height();
         settings.gap_size = 10;
@@ -340,6 +388,7 @@ SUITE(suite_draw)
                         RUN_TEST(test_layout_from_notification_no_icon);
                         RUN_TEST(test_calculate_dimensions_height_no_gaps);
                         RUN_TEST(test_calculate_dimensions_height_gaps);
+                        RUN_TEST(test_calculate_dimensions_height_min);
                         RUN_TEST(test_layout_render_no_gaps);
                         RUN_TEST(test_layout_render_gaps);
         });
