@@ -51,8 +51,7 @@ struct notification *test_notification(const char *name, gint64 timeout)
         struct notification *n = test_notification_uninitialized(name);
 
         notification_init(n);
-
-        n->format = "%s\n%b";
+        notification_replace_format(n, "%s\n%b");
 
         if (timeout != -1)
                 n->timeout = S2US(timeout);
