@@ -33,12 +33,8 @@ SUITE_EXTERN(suite_input);
 GREATEST_MAIN_DEFS();
 
 int main(int argc, char *argv[]) {
-        char *prog = realpath(argv[0], NULL);
-        if (!prog) {
-                fprintf(stderr, "Cannot determine actual path of test executable: %s\n", strerror(errno));
-                exit(1);
-        }
-        base = dirname(prog);
+        base = getenv("TESTDIR");
+        base = realpath(base ? base : "./test", NULL);
 
         /* By default do not print out warning messages, when executing tests.
          * But allow, if DUNST_TEST_LOG=1 is set in environment. */
@@ -48,8 +44,9 @@ int main(int argc, char *argv[]) {
 
 
         // initialize settings
-        char *config_path = g_strconcat(base, "/data/dunstrc.default", NULL);
-        load_settings(config_path);
+        char **configs = g_malloc0(2 * sizeof(char *));
+        configs[0] = g_strconcat(base, "/data/dunstrc.default", NULL);
+        load_settings(configs);
 
         GREATEST_MAIN_BEGIN();
         RUN_SUITE(suite_utils);
@@ -70,9 +67,8 @@ int main(int argc, char *argv[]) {
         RUN_SUITE(suite_rules);
         RUN_SUITE(suite_input);
 
-        base = NULL;
-        g_free(config_path);
-        free(prog);
+        settings_free(&settings);
+        g_strfreev(configs);
 
         // this returns the error code
         GREATEST_MAIN_END();
